#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define F_CPU 16000000L

#define VENDOR_RQ_SET_LED    0
#define VENDOR_RQ_GET_SWITCH 1
#define VENDOR_RQ_GET_LIGHT  2


uint8_t in_c(uint8_t pin)
{
    // กำหนดให้ขาที่ระบุของพอร์ท C ทำหน้าที่เป็นอินพุท
    DDRC &= ~(1<<pin);
    // ดึงให้ขามีลอจิก 1 หากขาถูกปล่อยลอยไว้ 
    PORTC |= (1<<pin);
    // อ่านสถานะลอจิกของขา
    return ((PINC & (1<<pin))>>pin);    
} 

void out_c(uint8_t pin, uint8_t val)
{
    // กำหนดให้ขาที่ระบุของพอร์ท C ทำหน้าที่เป็นเอาท์พุท
    DDRC |= 1<<pin;
    // เซ็ตลอจิกของขาเป็น 1 ถ้า val ไม่ใช่ศูนย์ ไม่เช่นนั้นเซ็ตให้เป็น 0
    if (val)
        PORTC |= 1<<pin;
    else
        PORTC &= ~(1<<pin);
}


int main()
{
	DDRC  |= 0b00000111;
	PORTC = 0b00000000;
	PORTC |= (1 << 3);
	DDRD  &= ~(0b00001000);
	PORTD = 0b00000000;

	usbInit();
	usbDeviceDisconnect();
	_delay_ms(300);     /* fake USB disconnect for > 250 ms */
	usbDeviceConnect();
	sei();              /* enable interrupts */
	while (1)           /* main event loop */
	{                
	   usbPoll();      /* คำสั่งนี้ต้องถูกเรียกอย่างน้อยที่สุดทุก ๆ 50ms */
	}
	return 0;
}

uint16_t read_adc(uint8_t channel)
{
        ADMUX = 0b01000000;
        ADMUX |= channel;
        ADCSRA = 0b11000110;
        while((ADCSRA & (1<<ADSC)))
                ;
        return ADCL + ADCH*256;

}

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
	usbRequest_t *rq = (void *)data;
	static uchar dataBuffer[2];  /* ข้อมูลนี้ต้องไม่ถูกเขียนทับหลังจาก usbFunctionSetup รีเทิร์น */

    /* ประมวลผลตามหมายเลขคำสั่งที่อยู่ใน bRequest */
	if (rq->bRequest == VENDOR_RQ_SET_LED)
	{
		uint8_t led_no  = rq->wValue.bytes[0];
		uint8_t led_val = rq->wValue.bytes[1];
		out_c(led_no, led_val);
		return 0;
	}
	else if (rq->bRequest == VENDOR_RQ_GET_SWITCH)
	{
		dataBuffer[0] = !in_c(3); /* กลับลอจิกเพื่อให้ 0 = ปล่อย 1 = กด */	
		usbMsgPtr = dataBuffer;   /* ระบุว่าข้อมูลส่งกลับอยู่ใน dataBuffer */
		return 1;                 /* ความยาวข้อมูลส่งกลับเท่ากับ 1 */
	}
	else if (rq->bRequest == VENDOR_RQ_GET_LIGHT)
	{
		uint16_t a = read_adc(4);
		dataBuffer[0] = a/256;
		dataBuffer[1] = a%256;
		usbMsgPtr = dataBuffer;
		return 2;	
	}
	return 0;   /* ไม่รู้จักคำร้องขอ ไม่ส่งข้อมูลกลับ */
}


